@@ -60,13 +60,10 @@ class autocast:
         # Save previous state
         self.prev_enabled = getattr(autocast._local, 'enabled', False)
         self.prev_dtype = getattr(autocast._local, 'dtype', xp.float32)
-        self.prev_cache = getattr(autocast._local, 'cache', None)
         
         # Set new state
         autocast._local.enabled = self.enabled
         autocast._local.dtype = self.dtype
-        # Fresh per-context cast cache to avoid duplicate casts
-        autocast._local.cache = {}
         
         return self
 
@@ -75,7 +72,6 @@ class autocast:
         # Restore previous state
         autocast._local.enabled = self.prev_enabled
         autocast._local.dtype = self.prev_dtype
-        autocast._local.cache = self.prev_cache
 
     @classmethod
     def is_enabled(cls) -> bool:
@@ -87,11 +83,6 @@ class autocast:
         """Get the current autocast dtype."""
         return getattr(cls._local, 'dtype', xp.float32)
 
-    @classmethod
-    def get_cache(cls) -> Optional[dict]:
-        """Get the per-context cast cache (internal)."""
-        return getattr(cls._local, 'cache', None)
-
 
 def is_autocast_enabled() -> bool:
     """Check if autocast is currently enabled (convenience function)."""
@@ -100,4 +91,4 @@ def is_autocast_enabled() -> bool:
 
 def get_autocast_dtype():
     """Get current autocast dtype (convenience function).""" 
-    return autocast.get_autocast_dtype()
+    return autocast.get_autocast_dtype()
\ No newline at end of file
